
# Specification::Set 这个类在 Pod::Source::Aggregate 的 search 方法里调用了

module Pod
  class Specification
    # A Specification::Set is responsible of handling all the specifications of
    # a Pod. This class stores the information of the dependencies that required
    # a Pod in the resolution process.
    #
    # @note   The order in which the sets are provided is used to select a
    #         specification if multiple are available for a given version.
    #
    # @note   The set class is not and should be not aware of the backing store
    #         of a Source.
    #
    class Set

      # override original
      def specification
        unless highest_version_spec_path
          raise Informative, "Could not find the highest version for `#{name}`. "\
                             "This could be due to an empty #{name} directory in a local repository."
        end

        # 如果一个 source 不好用，就用另外的 source 试试
        versions.each do |version|
            paths = specification_paths_for_version(highest_version)
            paths.each do |path|
                begin
                  spec = Specification.from_file(path)
                  # 这里没办法区分 oc / swift 库，暂时手动处理。前期全部保守的，如果不知道版本，都用 source。
                  # 后期可以放开。那是如果引了原版的 swift 库，就 build 失败，要先去去打依赖的库。
                  if $CocoapodsSwiftBinarySourceResolver_conservative
                    next unless spec.instance_variable_get(:@bd_multiple_swift_binary)
                  end
                  return spec # the real out
                rescue => e
                  inner_info_exception = e
                  if e.kind_of? DSLError
                    inner_info_exception = e.underlying_exception
                  end
                  # @swift_binary_exception 是在二进制的 podfile 里设的
                  unless inner_info_exception.instance_variable_get(:@swift_binary_exception) == true
                     raise e
                  end
                end
            end
        end

        raise Informative, "Could not find the version for `#{name}`. "\
                             "This could be due to an empty #{name} directory in a local repository."
      end

    end

  end
end
